@@ -4,6 +4,8 @@ use thiserror::Error;
 /* local use */
 use crate::utils;
 
+use std::string::String;
+
 #[derive(Debug, Error)]
 pub enum Error {
     #[error(
@@ -17,9 +19,6 @@ pub enum Error {
     #[error("Could not detect file format of '{filename:}'. File name should contains .fasta, .fa, .fastq, or .fq.")]
     UnableToDetectFileFormat { filename: String },
 
-    #[error("Error occured during reading of file '{filename:}'")]
-    ReadingError { filename: String },
-
     #[error(
         "Error during writing of sample/species file in format {format:?}"
     )]
