@@ -8,13 +8,59 @@ use std::fs::File;
 use std::io;
 use std::path::PathBuf;
 
-extern crate indicatif;
+extern crate chrono;
+extern crate fern;
 extern crate niffler;
 
 use crate::error;
 use anyhow::{anyhow, Context, Result};
 use bio::io::{fasta, fastq};
-use indicatif::{ProgressBar, ProgressStyle};
+use fern::colors::ColoredLevelConfig;
+
+pub fn setup_logging(quiet: bool) -> Result<(), fern::InitError> {
+    let colors = ColoredLevelConfig::default();
+    let mut base_config = fern::Dispatch::new();
+
+    base_config = match quiet {
+        // if user required quietness let only output warning messages
+        // or messages more severe than warnings
+        true => base_config.level(log::LevelFilter::Warn),
+        // if quietness is not specified which implies verbosity is allowed
+        // output
+        false => base_config.level(log::LevelFilter::Debug),
+    };
+
+    // Separate file config so we can include year, month and day in file logs
+    let file_config = fern::Dispatch::new()
+        .format(|out, message, record| {
+            out.finish(format_args!(
+                "{}[{}][{}] {}",
+                chrono::Local::now().format("[%Y-%m-%d][%H:%M:%S]"),
+                record.target(),
+                record.level(),
+                message
+            ))
+        })
+        .chain(fern::log_file("sabreur.log")?);
+
+    let stdout_config = fern::Dispatch::new()
+        .format(move |out, message, record| {
+            out.finish(format_args!(
+                "[{}][{}] {}",
+                chrono::Local::now().format("%H:%M:%S"),
+                colors.color(record.level()),
+                message
+            ))
+        })
+        .chain(io::stdout());
+
+    base_config
+        .chain(file_config)
+        .chain(stdout_config)
+        .apply()?;
+
+    Ok(())
+}
 
 pub fn create_relpath_from(input: Vec<&str>) -> Result<PathBuf> {
     let mut path = PathBuf::from("");
@@ -192,8 +238,8 @@ pub fn write_fa<'a>(
 ) -> Result<()> {
     let handle = niffler::get_writer(Box::new(file), compression, level)?;
 
-    let mut writer = fasta::Writer::new(handle);
-    let _write_res = writer.write_record(&record)?;
+    let mut writer = fasta::Writer::with_capacity(16264, handle);
+    let _write_res = writer.write_record(record)?;
 
     Ok(())
 }
@@ -219,8 +265,8 @@ pub fn write_fq<'a>(
     let handle = niffler::get_writer(Box::new(file), compression, level)
         .with_context(|| anyhow!("Could not get file writer"))?;
 
-    let mut writer = fastq::Writer::new(handle);
-    let _write_res = writer.write_record(&record).with_context(|| {
+    let mut writer = fastq::Writer::with_capacity(16264, handle);
+    let _write_res = writer.write_record(record).with_context(|| {
         error::Error::CantWriteFile {
             filename: "output file".to_string(),
         }
@@ -282,10 +328,7 @@ pub fn se_fa_demux<'a>(
     nb_records: &'a mut HashMap<&'a [u8], i32>,
 ) -> Result<(&'a mut HashMap<&'a [u8], i32>, bool)> {
     // Get fasta file reader and compression mode
-    let (reader, mut compression) =
-        read_file(file).with_context(|| error::Error::ReadingError {
-            filename: file.to_string(),
-        })?;
+    let (reader, mut compression) = read_file(file)?;
 
     // Get records
     let mut records = fasta::Reader::new(reader).records();
@@ -305,24 +348,7 @@ pub fn se_fa_demux<'a>(
         compression = format;
     }
 
-    // Progress bar
-    let pb = ProgressBar::new_spinner();
-    pb.enable_steady_tick(120);
-    pb.set_style(
-        ProgressStyle::default_spinner()
-            .tick_strings(&[
-                "▹▹▹▹▹",
-                "▸▹▹▹▹",
-                "▹▸▹▹▹",
-                "▹▹▸▹▹",
-                "▹▹▹▸▹",
-                "▹▹▹▹▸",
-                "▪▪▪▪▪",
-            ])
-            .template("[INFO] {spinner:.green} {msg}"),
-    );
     while let Some(Ok(record)) = records.next() {
-        pb.set_message("Demultiplexing...");
         // Match sequence and barcode with mismatch
         // and return matched barcode. We first use
         // let iter = my_vec.iter() to further stop
@@ -359,7 +385,6 @@ pub fn se_fa_demux<'a>(
             })?;
         }
     }
-    pb.finish_with_message("Done demultiplexing");
     Ok((nb_records, is_unk_empty))
 }
 
@@ -391,10 +416,7 @@ pub fn se_fq_demux<'a>(
     nb_records: &'a mut HashMap<&'a [u8], i32>,
 ) -> Result<(&'a mut HashMap<&'a [u8], i32>, bool)> {
     // Get fastq file reader and compression mode
-    let (reader, mut compression) =
-        read_file(file).with_context(|| error::Error::ReadingError {
-            filename: file.to_string(),
-        })?;
+    let (reader, mut compression) = read_file(file)?;
 
     // Get records
     let mut records = fastq::Reader::new(reader).records();
@@ -414,24 +436,7 @@ pub fn se_fq_demux<'a>(
         compression = format;
     }
 
-    // Progress bar
-    let pb = ProgressBar::new_spinner();
-    pb.enable_steady_tick(120);
-    pb.set_style(
-        ProgressStyle::default_spinner()
-            .tick_strings(&[
-                "▹▹▹▹▹",
-                "▸▹▹▹▹",
-                "▹▸▹▹▹",
-                "▹▹▸▹▹",
-                "▹▹▹▸▹",
-                "▹▹▹▹▸",
-                "▪▪▪▪▪",
-            ])
-            .template("[INFO] {spinner:.green} {msg}"),
-    );
     while let Some(Ok(record)) = records.next() {
-        pb.set_message("Demultiplexing...");
         let mut iter = my_vec.iter();
         let matched_barcode =
             iter.find(|&&x| bc_cmp(x, &record.seq()[..bc_len], mismatch));
@@ -464,7 +469,6 @@ pub fn se_fq_demux<'a>(
             })?;
         }
     }
-    pb.finish_with_message("Done demultiplexing");
     Ok((nb_records, is_unk_empty))
 }
 
@@ -484,15 +488,9 @@ pub fn pe_fa_demux<'a>(
     nb_records: &'a mut HashMap<&'a [u8], i32>,
 ) -> Result<(&'a mut HashMap<&'a [u8], i32>, String)> {
     // Get fasta files reader and compression modes
-    let (forward_reader, mut compression) =
-        read_file(forward).with_context(|| error::Error::ReadingError {
-            filename: forward.to_string(),
-        })?;
+    let (forward_reader, mut compression) = read_file(forward)?;
 
-    let (reverse_reader, _compression) =
-        read_file(reverse).with_context(|| error::Error::ReadingError {
-            filename: reverse.to_string(),
-        })?;
+    let (reverse_reader, _compression) = read_file(reverse)?;
 
     // Get records
     let mut forward_records = fasta::Reader::new(forward_reader).records();
@@ -514,24 +512,7 @@ pub fn pe_fa_demux<'a>(
         compression = format;
     }
 
-    // Progress bar
-    let pb = ProgressBar::new_spinner();
-    pb.enable_steady_tick(120);
-    pb.set_style(
-        ProgressStyle::default_spinner()
-            .tick_strings(&[
-                "▹▹▹▹▹",
-                "▸▹▹▹▹",
-                "▹▸▹▹▹",
-                "▹▹▸▹▹",
-                "▹▹▹▸▹",
-                "▹▹▹▹▸",
-                "▪▪▪▪▪",
-            ])
-            .template("[INFO] {spinner:.green} {msg}"),
-    );
     while let Some(Ok(f_rec)) = forward_records.next() {
-        pb.set_message("Demultiplexing forward file...");
         let mut iter = my_vec.iter();
         let matched_barcode =
             iter.find(|&&x| bc_cmp(x, &f_rec.seq()[..bc_len], mismatch));
@@ -566,7 +547,6 @@ pub fn pe_fa_demux<'a>(
     }
 
     while let Some(Ok(r_rec)) = reverse_records.next() {
-        pb.set_message("Demultiplexing reverse file...");
         let mut iter = my_vec.iter();
         let matched_barcode =
             iter.find(|&&x| bc_cmp(x, &r_rec.seq()[..bc_len], mismatch));
@@ -599,7 +579,6 @@ pub fn pe_fa_demux<'a>(
             })?;
         }
     }
-    pb.finish_with_message("Done");
     let final_str = format!("{}{}", unk1_empty, unk2_empty);
     Ok((nb_records, final_str))
 }
@@ -620,15 +599,9 @@ pub fn pe_fq_demux<'a>(
     nb_records: &'a mut HashMap<&'a [u8], i32>,
 ) -> Result<(&'a mut HashMap<&'a [u8], i32>, String)> {
     // Get fasta files reader and compression modes
-    let (forward_reader, mut compression) =
-        read_file(forward).with_context(|| error::Error::ReadingError {
-            filename: forward.to_string(),
-        })?;
+    let (forward_reader, mut compression) = read_file(forward)?;
 
-    let (reverse_reader, _compression) =
-        read_file(reverse).with_context(|| error::Error::ReadingError {
-            filename: reverse.to_string(),
-        })?;
+    let (reverse_reader, _compression) = read_file(reverse)?;
 
     // Get records
     let mut forward_records = fastq::Reader::new(forward_reader).records();
@@ -650,25 +623,7 @@ pub fn pe_fq_demux<'a>(
         compression = format;
     }
 
-    // Progress bar
-    let pb = ProgressBar::new_spinner();
-    pb.enable_steady_tick(120);
-    pb.set_style(
-        ProgressStyle::default_spinner()
-            .tick_strings(&[
-                "▹▹▹▹▹",
-                "▸▹▹▹▹",
-                "▹▸▹▹▹",
-                "▹▹▸▹▹",
-                "▹▹▹▸▹",
-                "▹▹▹▹▸",
-                "▪▪▪▪▪",
-            ])
-            .template("[INFO] {spinner:.green} {msg}"),
-    );
-
     while let Some(Ok(f_rec)) = forward_records.next() {
-        pb.set_message("Demultiplexing forward file...");
         let mut iter = my_vec.iter();
         let matched_barcode =
             iter.find(|&&x| bc_cmp(x, &f_rec.seq()[..bc_len], mismatch));
@@ -703,7 +658,6 @@ pub fn pe_fq_demux<'a>(
     }
 
     while let Some(Ok(r_rec)) = reverse_records.next() {
-        pb.set_message("Demultiplexing reverse file...");
         let mut iter = my_vec.iter();
         let matched_barcode =
             iter.find(|&&x| bc_cmp(x, &r_rec.seq()[..bc_len], mismatch));
@@ -737,7 +691,6 @@ pub fn pe_fq_demux<'a>(
         }
     }
     let final_str = format!("{}{}", unk1_empty, unk2_empty);
-    pb.finish_with_message("Done");
     Ok((nb_records, final_str))
 }
 
